# Licensed Materials - Property of IBM
# TIVOCIT00
# Copyright IBM Corp. 2005-2007. All Rights Reserved.
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule contract with
# IBM Corp.

# Version: src/enabler/VMWare/linux/retrieve.pl, cit, cit_2.6, 20090422v2.6
# Build date: 09/04/22 16:37:31

# Input:
# 6 parameters: node id, node manufact, node product, node version, node type,node capacity
# 7 parameters: node id, node manufact, node product, node version, node type, node capacity, verbose flag

my $OUTFILE;
my $verbose;

sub debug($) {
    if ($verbose) {
        if (open(OUTFILE, ">>retr_out.txt")) {
            my($sec, $min, $hour) = (localtime)[0, 1, 2];
            printf OUTFILE "[%02d:%02d:%02d][%s:%s] %s\n", $hour, $min, $sec, getppid, $$, shift;
            close OUTFILE;
        }
    }
}

sub do_exit($)
{
    my $exit_code = shift;
    
    my($sec, $min, $hour, $day, $month, $year) = (localtime)[ 0, 1, 2, 3, 4, 5 ];
    my $msg = sprintf "=========== FINISHED on %02d/%02d/%d %02d:%02d:%02d == exit code = %d ===========\n",$month+1,$day,1900+$year,$hour,$min,$sec,$exit_code;
    debug $msg;
    
    exit($exit_code);
}


my $node_id;
my $node_man;
my $node_prod;
my $node_ver;
my $node_type;
my $node_capacity;

BEGIN
{
    if ((@ARGV ne 6) && (@ARGV ne 7)) {
        do_exit(5);
    }

    if (@ARGV == 6) {
      ($node_id,$node_man,$node_prod,$node_ver,$node_type, $node_capacity) = @ARGV;
    }
    if (@ARGV == 7) {
        ($node_id, $node_man,$node_prod,$node_ver,$node_type,$node_capacity) = @ARGV;
        $verbose = 1;
    }
    
    $node_id =~ s/^"(.*)"$/$1/;
    $node_man =~ s/^"(.*)"$/$1/;
    $node_prod =~ s/^"(.*)"$/$1/;
    $node_ver =~ s/^"(.*)"$/$1/;
    $node_type =~ s/^"(.*)"$/$1/;
    $node_capacity =~ s/^"(.*)"$/$1/;
    
    if ($verbose) {
        if (open(OUTFILE, ">>retr_out.txt"))
        {
            my($sec, $min, $hour, $day, $month, $year) = (localtime)[ 0, 1, 2, 3, 4, 5 ];
            my $msg;
            
            if (!defined($ENV{'ENABLER_SECOND_TRY'}))
            {
                $msg = sprintf "==================== STARTED on %02d/%02d/%d %02d:%02d:%02d ====================\n",$month+1,$day,1900+$year,$hour,$min,$sec;
            }
            else
            {
                $msg = "Second try...";
            }
            
            debug $msg;
        }
        else
        {
            printf "WARNING! Failed to open log file (retr_out.txt)";
        }
    }

    ######### try to load VmPerl libraries ##########
    
    eval {
        require VMware::VmPerl;
        require VMware::VmPerl::VM;
        require VMware::VmPerl::Server;
        require VMware::VmPerl::ConnectParams;
    };

    if ($@)
    {
        #failed to load VmPerl
        
        if (!defined($ENV{'ENABLER_SECOND_TRY'}))
        {
            if (open CONFIG, '</etc/vmware/config')
            {
                my $line;
                while ($line = <CONFIG>)
                {
                    if ($line =~ /^\s*libdir\s*=\s*\"(.*)\"\s*$/)
                    {
                        $libdir = $1;
                        last;
                    }
                }

                close CONFIG;
                if (defined($libdir)) 
                {
                    chomp $libdir;
                    my $perl = $libdir . '/perl5/bin/perl';

                    if (-x $perl)
                    {
                        $ENV{'ENABLER_SECOND_TRY'} = 1;
                        exec $perl, '-I'.$libdir.'/perl5/site_perl/5.005', $0, @ARGV;
                    }
                }
            }
        }
        
        debug "ERROR - VmPerl libraries not found";
        do_exit(120);
    }
}


my $TimeOutPerVM = 5*60; # 5 minutes

pipe FROM_PARENT,TO_CHILD; select((select(TO_CHILD), $| = 1)[0]);   # autoflush
pipe FROM_CHILD,TO_PARENT; select((select(TO_PARENT), $| = 1)[0]);   # autoflush

my $child_pid = undef;

$SIG{INT} = \&handle_SIGINT;


sub setvmvar {
	debug "";
	debug "ENTRY: setvmwar";
    my($vm, $cfg_path, $key, $value) = @_;
    my $rc;
	debug "calling get_guest_info($key)";
    my $get = $vm->get_guest_info($key);
	
    debug "got: $get, value to set: $value";
    if ($get ne $value)
    {
		my $left=10;
		sleep(1);
		debug "using command: /usr/bin/vmware-cmd  \"$cfg_path\"  \"setguestinfo\" \"$key\" \"$value\"";
		@syscmd = ("/usr/bin/vmware-cmd  \"$cfg_path\"  \"setguestinfo\" \"$key\" \"$value\" > /dev/null 2>&1");
		my $sysrc = system(@syscmd);
		debug "sysrc: $sysrc";	
		if ($sysrc == 0) {
			sleep(5);
			debug "OK: sysrc was 0, calling get_guest_info($key)";
			$get = $vm->get_guest_info($key);
			debug "got: $get, expected: $value";	
			while ($value ne $get)
	        {
				sleep(5);
	            $get = $vm->get_guest_info($key);
				debug "got: $get, expected: $value";   
	            last if ($left == 0); $left--;
	        }
			if ($value eq $get) {
				debug "OK: set key $key to '$value' via vmware-cmd command successfully";
			}
			else
	        {
	            debug "WARNING: Failed to set key $key from '$get' to '$value'";
			};
		} else {
			debug "WARNING: set key $key to '$value' via vmware-cmd command failed!!";
		};
    }
    else
    {
        debug "OK: Key $key already set to '$value'";
    }
	debug "EXIT: setvmwar";
	debug "";
}



my $vm_capacity = 1; # if running on a ESX server this value will be changed later


# Get data from PHYSICAL_PROCESSOR environmental variable
my $physical_processor_data = $ENV{PHYSICAL_PROCESSOR};
debug "PHYSICAL_PROCESSOR = '$physical_processor_data'\n";

my $vm_version = VMware::VmPerl::version();
debug "VMware::VmPerl::version() returned: $vm_version";
if (! $vm_version) {
   debug "Could not run Perl VMware::VmPerl module\n";
   exit(255);
} 
debug "Connecting to server...";

my $hostname = undef;
my $port = 0;
my $username = undef;
my $password = undef;

# Establish a persistent connection with server
my $server = VMware::VmPerl::Server::new();

# Create a ConnectParams object
my $connect_params = VMware::VmPerl::ConnectParams::new($hostname, $port, $username, $password);

if (!$server->connect($connect_params)) {

   # Could not connect to server
    my ($error_number, $error_string) = $server->get_last_error();
    debug "ERROR($error_number): $error_string\n";
    debug "ERROR: Could not connect to server\n";
    
    do_exit(7);
}

debug "Connected to server. Getting list of VMs...";

# Enumerate all VMs
# Obtain a list containing every config file path registered with the server.
my @list = $server->registered_vm_names();
if (!@list) {

    # Could not get list of VMs from server
    my ($error_number, $error_string) = $server->get_last_error();
    debug "ERROR($error_number): $error_string\n";

    debug "ERROR: Could not get list of VMs from server\n";
    
    do_exit(6);
}

debug "Got list of VMs.";


if ($child_pid = fork)
{
    debug "Started child pid=$child_pid\n";
    
    my $vm_path = undef;
    my @vm_list = @list;

    while ($vm_path = shift @vm_list)
    {
        my $cmd = sprintf "%s:%s:%s:%s:%s:%s:%s\n", $vm_path,$node_id,$node_man,$node_prod,$node_ver,$node_type, $node_capacity;
        
        debug "Sending '$cmd'";
        
        printf TO_CHILD $cmd;
        
        my $rin='';
        vec($rin, fileno(FROM_CHILD), 1) = 1;
        my $nfound = select($rin,undef,undef,$TimeOutPerVM);
        
        if ($nfound)
        {
            debug "Child done vm";
            my $answer = <FROM_CHILD>;
        }
        else
        {
            debug "Child timeouted";
            kill 9, $child_pid;
            waitpid(-1,0);

            if ($child_pid = fork)
            {
                debug "Started child pid=$child_pid\n";
            }
            else
            {
                child_routine();
            }
        }
    }
        
    kill 9, $child_pid;
    waitpid(-1,0);

    debug "Child ended\n";
   

# Destroys the server object, thus disconnecting from the server.
undef $server;
    
debug "Disconnected from server.\n";

do_exit(0);

}
else #child
{
    child_routine();
}

sub child_routine 
{
    debug "Started";
    
    my $connect_params = VMware::VmPerl::ConnectParams::new();
    
    my $args;
    while ($args = <FROM_PARENT>)
    {
        chop $args;
        
        my ($vm_path,$node_id,$node_man,$node_prod,$node_ver,$node_type, $node_capacity) = split(/:/, $args);
        
        debug "Got vm_path='$vm_path' node_id='$node_id' node_man='$node_man' node_prod='$node_prod' node_ver='$node_ver' node_type='$node_type' node_capacity='$node_capacity'\n";
        
        debug "Connecting to VM: ".$vm_path;
      
        my $vm = VMware::VmPerl::VM::new();

        if (!$vm->connect($connect_params, $vm_path)) {
        
            # Could not connect to vm
            debug "WARNING! Could not connect to virtual machine: $vm_path. Probably powered down.";
        }
        else
        {
            debug "Connected to VM.";
            
            my $vm_uuid = $vm->get_config("uuid.bios");
            if (!defined($vm_uuid)) {
                $vm_uuid = "NO UUID";
            }
            
            my $vm_name = $vm->get_config("displayName");
            if (!defined($vm_name)) {
                $vm_name = "NO NAME ";
            }
            $vm_name = $vm_name . "_" . $vm_uuid; #makes vm_name unique

            debug "OK: Virtual machine ID: ".$vm_name;

            # Check if this VM runs on a ESX Server
            if ($vm->get_product_info(VM_PRODINFO_PRODUCT) == VM_PRODUCT_ESX) {
            
                # Change VM Capacity
                # to reflect VM configuration on ESX Server
                my $cpu_max = $vm->get_resource("cpu.max");
                debug "OK: vm cpu_max=".$cpu_max;
                
                $vm_capacity = $cpu_max / 100;
                debug "OK: vm_capacity=".$vm_capacity;

            }

            # On ESX Server this information is retrieved
            # from VM and if this VM is down the value printed
            # can be missleading
            # if ($verbose) {
            #    printf OUTFILE "OK: Virtual machine capacity: %d\n\n", $vm_capacity;
            # }

            if (($vm->get_execution_state() == VM_EXECUTION_STATE_ON) || ($vm->get_execution_state() == 1)) {
            
                debug "Transfering data to machine: ".$vm_name;

                setvmvar($vm, $vm_path, "CIT_NODE_ID", $node_id);
                setvmvar($vm, $vm_path, "CIT_NODE_MAN", $node_man);
                setvmvar($vm, $vm_path, "CIT_NODE_PROD", $node_prod);
                setvmvar($vm, $vm_path, "CIT_NODE_VER", $node_ver);
                setvmvar($vm, $vm_path, "CIT_NODE_TYPE", $node_type);
                setvmvar($vm, $vm_path, "CIT_NODE_CAPACITY", $node_capacity);
                setvmvar($vm, $vm_path, "CIT_VM_ID", $vm_name);
                setvmvar($vm, $vm_path, "CIT_VM_CAPACITY", $vm_capacity);
                setvmvar($vm, $vm_path, "CIT_PHYSICAL_PROCESSOR", $physical_processor_data);

                debug "Transfer done.";

            }
            else {
                  debug "WARNING: Machine not running: $vm_path\n";
            }

        }#connected to vm
        
        # Destroys the virtual machine object, thus disconnecting from the virtual machine.
        undef $vm;

        debug "Disconnected from VM.\n";
        
        print TO_PARENT "\n";
        
    }

    debug "Done";
    
    do_exit(0);
}


sub handle_SIGINT 
{
    debug "Caught SIGINT";
    
    if (defined $child_pid)
    {
        debug "Killing child";
        kill 9, $child_pid;
    }
    
    close TO_CHILD;
    close FROM_CHILD;
    close TO_PARENT;
    close FROM_PARENT;
    
    do_exit(8);
}
